/*
 * Licensed Materials - Property of IBM
 *
 * Copyright IBM Corporation 2009. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA DP Schedule Contract with IBM Corp.
 */
	
egl.defineWidget(
	'dojo.widgets', 'DojoGrid',
	'dojo.widgets', 'DojoBase',
	'div',
{
	"constructor" : function() {
		this.startup = false;
		this.headerBehaviors = [];
		this.behaviors = [];
		this.data = [];
		dojo.require("dojox.grid.DataGrid");
		dojo.require("dojo.data.ItemFileReadStore");
	},
	"createDojoWidget" : function(parent) {
		this.setInnerHTML("");
		var eglWidget = this;
		this.dojoColumns = this.createColumns();
		this.dojoStore = new dojo.data.ItemFileReadStore({ data: this.createData(this.data, this.dojoColumns) });
		// 
	    this.dojoWidget = new dojox.grid.DataGrid( {
			id: eglWidget.id, 
			structure: eglWidget.dojoColumns,
			store : eglWidget.dojoStore,
			autoHeight: true,
			autoWidth: true,
			splitter: false
		}, parent);
	    
	    var runBehaviorsFunc = function() {
	    	setTimeout(function() {
	    		if (!eglWidget.startup) { eglWidget.runBehaviors(); }
	    		eglWidget.startup = true;
		    },1);
	    };
	    
	    var runPostBehaviorsFunc = function() {
	    	setTimeout(function() { eglWidget.runBehaviors(); },1);
	    };
	    // postrender and adaptWidth is called multiple times from dojo, hence the startup flag in runBehaviorsFunc
	    this.dojoWidget.connect(this.dojoWidget, "postrender", runBehaviorsFunc );
	    // sort and onResizeColumn are called only once from within Dojo
	    this.dojoWidget.connect(this.dojoWidget, "sort", runPostBehaviorsFunc);
	    this.dojoWidget.connect(this.dojoWidget, "onResizeColumn", runPostBehaviorsFunc );
	    //
	    eglWidget.dojoWidget.render();
	    eglWidget.dojoWidget.startup();
    },
    "setData" : function (data) {
    	this.startup = false;
    	if (!data)
    		throw "Illegal argument: DojoGrid data should be an array of records or dictionaries, not null.";
    	if (data[0] && data.eze$$getChildVariables && (data[0].eze$$getChildVariables && !data[0].eze$$typename))
    			throw "Illegal argument: DojoGrid data should be an array of records or dictionaries, not a nested array.";
        egl.dojo.widgets.DojoBase.prototype.setData.call(this, data);
    },
    "runBehaviors" : function() {
    	this.addBehaviors(this.behaviors, "td", this.data.length);
    	this.addBehaviors(this.headerBehaviors, "th", 1);	
    	this.fixSizes();
    },
    "fixSizes": function() {
    	var tables = this.dojoWidget.domNode.getElementsByTagName("tbody");
	    var w = 0, h = 0, headerHeight;
	    for (var n=1; n<tables.length; n++) {
	    	if (tables[n].className != "EglRuiTbody") {
		    	h += egl.getHeightInPixels(tables[n]) + 1;
		    	w = Math.max(w, egl.getWidthInPixels(tables[n]) + 1);
	    	}
	    }
	    var divs = this.dojoWidget.domNode.getElementsByTagName("div");
	    for (var n=0; n<divs.length; n++) { 
	    	var div = divs[n];
    		if (this.isGridDiv(div.className)) {
	    		div.style.height = egl.toPX(h); 
	    		div.style.width = egl.toPX(w);
	    	}
	    }
	    this.setWidth(w);  
	    this.setHeight(h + egl.getHeightInPixels(tables[0]) + 1);
	    //if (w<10)
	    	//this.showError(w, h, headerWidth);
    },
    "isGridDiv" : function(name) {
    	switch (name) {
	    	case "dojoxGrid-master-view" :
	    	case "dojoxGridMasterView" :
	    	case "dojoxGrid" : 
	    	case "dojoxGrid-view" :
	    	case "dojoxGridView" :
	    	case "dojoxGrid-content" :
	    	case "dojoxGridContent":
			case "dojoxGrid-scrollbox":
			case "dojoxGridScrollbox":
				return true;
    	}
		return false;
    },
    "showError": function(w, h) {
    	var s = "This is an empty DojoGrid. Nothing was rendered.";
    	if (this.data.length > 0) {
    		s += "<p>There are "+this.data.length+" records and "+this.columns.length+" columns."
	    	s += "<p>Columns:";
    		var missing = null;
    		var value = this.data.length>0 ? this.data[0] : {};
    		for (var n=0; n<this.dojoColumns.length; n++) {
    			s += " "+n+":\""+this.dojoColumns[n].field + "\""
    			if (!(this.dojoColumns[n].field in value))
    				missing = missing || this.dojoColumns[n].field;
    		}
    		s += "<p>Data: ["+this.data.length+"] {";
    		for (f in value)
				if (egl.isUserField(value, f))
					s += " \""+f+"\"";
    		s += "}";
    		s += "<p>Grid width = "+w + ". Grid height = "+h;
    		if (missing)
    			s += "<br>Check your column definitions. Missing data field: "+missing;
    		else
    			s += "<p><b>Internal Error</b>. This appears to be a race condition. " + 
    				"Please report this problem to the proper authorities.<br>" +
    				"Browser: " + window.navigator.userAgent;
    	}
	    this.setWidth(600);  
  		this.setHeight(400);
    	this.setInnerHTML(s);
    	egl.setEnableTextSelection(true);
    },
    "createData": function(data, columns) {
		var cells = {identifier: '', label: '', items: [{ }]};
		for (var row=0; row < data.length; row++) {
			cells.items[row] = {};
		    for (var column=0; column < columns.length; column++) {
		    	var value = ""+egl.unboxAny(data[row][columns[column].field]);
		    	if (typeof(value) == "undefined")
		    		value = "&nbsp;";
		    	var num = Number(value);
		    	if (!isNaN(num))
		    		value = num;
		    	var field = columns[column].field;
	    		cells.items[row][field] = value;
		    }
		}
		return cells;
    },
    "createColumns" : function() {
		this.columns = this.columns || [];
		var eglWidget = this;
		var result = [];
	    if (!this.columns.length && this.data.length) {
			for (f in this.data[0])
				if (egl.isUserField(this.data[0], f))
					result.push({name: f, field: f});
			this.columns = result;
	    }
	    else {
			for (var n=0; n < this.columns.length; n++) {	    	
				var column = this.columns[n];
				var name = column.displayName || column.name;
				name = name.replace(/ /g, "&nbsp;");
				result.push({
					field: column.name,
					name: name,
					width: column.width ? egl.toPX(column.width) : '100px'
				});
			}
	    }
		return result;
    },
    "addBehaviors" : function(behaviors, cellType, rowCount) {
    	var cells = [].appendAll(this.dojoWidget.domNode.getElementsByTagName(cellType));
    	for (var n=0; n<behaviors.length; n++) {
    		for (var row=0; row<rowCount; row++) {
	    		for (var column=0; column < this.columns.length; column++) {
	    			var cell = cells[row*this.columns.length + column];
    				try {
	    				behaviors[n](
	    					this, 
	    					egl.createWidget(cell), 
	    				    this.data[row], 
	    				    row+1, 
	    				    this.columns[column]
	    				);
	    			}
	    			catch (e) {
	    				var msg = "Error while running behavior for " +	(rowCount ? "row="+row : "header") + 
	    					", column=\""+this.columns[column].name+"\", cell="+ (row*this.columns.length + column)+
	    					" of " + cells.length;
	    				if (row) {
	    					msg += " row data is: <ul><table cellPadding=3px border=1>"
	    						for (var column=0; column < this.columns.length; column++) {
	    							msg += "<tr><td>" + this.columns[column].displayName + "</td>";
	    							msg += "<td>" + this.data[row][this.columns[column].name] + "</td></tr>";
	    						}
	    					msg += "</table></ul>Cancelling the rest of the behaviors for this grid.";
	    				}
	    				egl.printError(msg, e);
	    				return;
	    			}
	    		}
        	}
    	}
    },
	"addRow" : function( rowData ) {
		this.dojoWidget.addRow(this.createData(rowData));
	},
	"removeRow" : function( rowIndex ) {
		this.dojoWidget.selection.setSelected(rowIndex,true);
		this.dojoWidget.removeSelectedRows();		
	},
	"destroy" : function() {
	}
});